"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FastColor = void 0;
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
class FastColor {
  constructor(input) {
    /**
     * Red, R in RGB
     */
    (0, _defineProperty2.default)(this, "r", void 0);
    /**
     * Green, G in RGB
     */
    (0, _defineProperty2.default)(this, "g", void 0);
    /**
     * Blue, B in RGB
     */
    (0, _defineProperty2.default)(this, "b", void 0);
    /**
     * Alpha/Opacity, A in RGBA/HSLA
     */
    (0, _defineProperty2.default)(this, "a", void 0);
    // HSV privates
    (0, _defineProperty2.default)(this, "_h", void 0);
    (0, _defineProperty2.default)(this, "_s", void 0);
    (0, _defineProperty2.default)(this, "_l", void 0);
    (0, _defineProperty2.default)(this, "_v", void 0);
    // intermedia variables to calculate HSL/HSV
    (0, _defineProperty2.default)(this, "_max", void 0);
    (0, _defineProperty2.default)(this, "_min", void 0);
    (0, _defineProperty2.default)(this, "_brightness", void 0);
    if (typeof input === 'string') {
      const trimed = input.trim();
      if (trimed[0] === '#') {
        this.fromHexString(trimed);
      } else if (trimed.startsWith('rgb')) {
        this.fromRgbString(trimed);
      } else if (trimed.startsWith('hsl')) {
        this.fromHslString(trimed);
      }
    } else if ('r' in input && 'g' in input && 'b' in input) {
      this.r = input.r;
      this.g = input.g;
      this.b = input.b;
      this.a = typeof input.a === 'number' ? input.a : 1;
    } else if ('l' in input && 'h' in input && 's' in input) {
      this.fromHsl(input);
    } else if ('v' in input && 'h' in input && 's' in input) {
      this.fromHsv(input);
    } else {
      throw new Error('@ant-design/fast-color: unsupported input ' + JSON.stringify(input));
    }
  }

  /**
   * Hue, H in HSL/HSV
   */
  get h() {
    return this.getHue();
  }

  /**
   * Saturation, S in HSL/HSV
   */
  get s() {
    return this.getSaturation();
  }

  /**
   * Lightness, L in HSL
   */
  get l() {
    return this.getLightness();
  }

  /**
   * Value, V in HSV
   */
  get v() {
    return this.getValue();
  }
  get isValid() {
    return typeof this.r === 'number' && this.r >= 0 && this.r <= 255 && typeof this.g === 'number' && this.g >= 0 && this.g <= 255 && typeof this.b === 'number' && this.b >= 0 && this.b <= 255 && typeof this.a === 'number' && this.a >= 0 && this.a <= 1;
  }
  clone() {
    return new FastColor(this);
  }
  equals(other) {
    return this.r === other.r && this.g === other.g && this.b === other.b && this.a === other.a;
  }
  darken(amount = 10) {
    const h = this.getHue();
    const s = this.getSaturation();
    let l = this.getLightness() - amount / 100;
    if (l < 0) {
      l = 0;
    }
    return new FastColor({
      h,
      s,
      l,
      a: this.a
    });
  }
  lighten(amount = 10) {
    const h = this.getHue();
    const s = this.getSaturation();
    let l = this.getLightness() + amount / 100;
    if (l > 1) {
      l = 1;
    }
    return new FastColor({
      h,
      s,
      l,
      a: this.a
    });
  }

  /**
   * Mix the color with pure white, from 0 to 100.
   * Providing 0 will do nothing, providing 100 will always return white.
   */
  tint(amount = 10) {
    return this.mix({
      r: 255,
      g: 255,
      b: 255,
      a: 1
    }, amount);
  }

  /**
   * Mix the color with pure black, from 0 to 100.
   * Providing 0 will do nothing, providing 100 will always return black.
   */
  shade(amount = 10) {
    return this.mix({
      r: 0,
      g: 0,
      b: 0,
      a: 1
    }, amount);
  }

  /**
   * Mix the current color a given amount with another color, from 0 to 100.
   * 0 means no mixing (return current color).
   */
  mix(color, amount = 50) {
    const rgb1 = this.toRgb();
    const rgb2 = new FastColor(color).toRgb();
    const p = amount / 100;
    const rgba = {
      r: (rgb2.r - rgb1.r) * p + rgb1.r,
      g: (rgb2.g - rgb1.g) * p + rgb1.g,
      b: (rgb2.b - rgb1.b) * p + rgb1.b,
      a: (rgb2.a - rgb1.a) * p + rgb1.a
    };
    return new FastColor(rgba);
  }
  getAlpha() {
    return this.a;
  }

  /**
   * Returns the perceived luminance of a color, from 0-1.
   * @see http://www.w3.org/TR/2008/REC-WCAG20-20081211/#relativeluminancedef
   */
  getLuminance() {
    let R;
    let G;
    let B;
    const RsRGB = this.r / 255;
    const GsRGB = this.g / 255;
    const BsRGB = this.b / 255;
    if (RsRGB <= 0.03928) {
      R = RsRGB / 12.92;
    } else {
      R = Math.pow((RsRGB + 0.055) / 1.055, 2.4);
    }
    if (GsRGB <= 0.03928) {
      G = GsRGB / 12.92;
    } else {
      G = Math.pow((GsRGB + 0.055) / 1.055, 2.4);
    }
    if (BsRGB <= 0.03928) {
      B = BsRGB / 12.92;
    } else {
      B = Math.pow((BsRGB + 0.055) / 1.055, 2.4);
    }
    return 0.2126 * R + 0.7152 * G + 0.0722 * B;
  }
  getHue() {
    if (typeof this._h === 'undefined') {
      const delta = this.max - this.min;
      if (delta === 0) {
        this._h = 0;
      } else {
        this._h = Math.round(60 * (this.r === this.max ? (this.g - this.b) / delta + (this.g < this.b ? 6 : 0) : this.g === this.max ? (this.b - this.r) / delta + 2 : (this.r - this.g) / delta + 4));
      }
    }
    return this._h;
  }
  getSaturation() {
    if (typeof this._s === 'undefined') {
      const delta = this.max - this.min;
      if (delta === 0) {
        this._s = 0;
      } else {
        this._s = delta / this.max;
      }
    }
    return this._s;
  }
  getLightness() {
    if (typeof this._l === 'undefined') {
      this._l = (this.max + this.min) / 510;
    }
    return this._l;
  }
  getValue() {
    if (typeof this._v === 'undefined') {
      this._v = this.max / 255;
    }
    return this._v;
  }
  isDark() {
    return this.getBrightness() < 128;
  }
  isLight() {
    return this.getBrightness() >= 128;
  }

  /**
   * Returns the perceived brightness of the color, from 0-255.
   * @see http://www.w3.org/TR/AERT#color-contrast
   */
  getBrightness() {
    if (typeof this._brightness === 'undefined') {
      this._brightness = (this.r * 299 + this.g * 587 + this.b * 114) / 1000;
    }
    return this._brightness;
  }
  onBackground(background) {
    const bg = new FastColor(background);
    const alpha = this.a + bg.a * (1 - this.a);
    return new FastColor({
      r: Math.round((this.r * this.a + bg.r * bg.a * (1 - this.a)) / alpha),
      g: Math.round((this.g * this.a + bg.g * bg.a * (1 - this.a)) / alpha),
      b: Math.round((this.b * this.a + bg.b * bg.a * (1 - this.a)) / alpha),
      a: alpha
    });
  }
  setAlpha(alpha) {
    this.a = alpha;
    return this;
  }
  toHexString() {
    let hex = '#';
    const rHex = (this.r || 0).toString(16);
    hex += rHex.length === 2 ? rHex : '0' + rHex;
    const gHex = (this.g || 0).toString(16);
    hex += gHex.length === 2 ? gHex : '0' + gHex;
    const bHex = (this.b || 0).toString(16);
    hex += bHex.length === 2 ? bHex : '0' + bHex;
    if (typeof this.a === 'number' && this.a >= 0 && this.a < 1) {
      const aHex = Math.round(this.a * 255).toString(16);
      hex += aHex.length === 2 ? aHex : '0' + aHex;
    }
    return hex;
  }
  toHsl() {
    return {
      h: this.h,
      s: this.s,
      l: this.l
    };
  }
  toHslString() {
    const h = this.h;
    const s = Math.round(this.s * 100);
    const l = Math.round(this.l * 100);
    return this.a !== 1 ? `hsla(${h},${s}%,${l}%,${this.a})` : `hsl(${h},${s}%,${l}%)`;
  }
  toHsv() {
    return {
      h: this.h,
      s: this.s,
      v: this.v,
      a: this.a
    };
  }
  toRgb() {
    return {
      r: this.r,
      g: this.g,
      b: this.b,
      a: this.a
    };
  }
  toRgbString() {
    return this.a !== 1 ? `rgba(${this.r},${this.g},${this.b},${this.a})` : `rgb(${this.r},${this.g},${this.b})`;
  }
  toString() {
    return this.toRgbString();
  }
  get max() {
    if (typeof this._max === 'undefined') {
      this._max = Math.max(this.r, this.g, this.b);
    }
    return this._max;
  }
  get min() {
    if (typeof this._min === 'undefined') {
      this._min = Math.min(this.r, this.g, this.b);
    }
    return this._min;
  }
  fromHexString(trimed) {
    if (trimed.length < 6) {
      // #rgb or #rgba
      this.r = parseInt(trimed[1] + trimed[1], 16);
      this.g = parseInt(trimed[2] + trimed[2], 16);
      this.b = parseInt(trimed[3] + trimed[3], 16);
      this.a = trimed[4] ? parseInt(trimed[4] + trimed[4], 16) / 255 : 1;
    } else {
      // #rrggbb or #rrggbbaa
      this.r = parseInt(trimed[1] + trimed[2], 16);
      this.g = parseInt(trimed[3] + trimed[4], 16);
      this.b = parseInt(trimed[5] + trimed[6], 16);
      this.a = trimed[8] ? parseInt(trimed[7] + trimed[8], 16) / 255 : 1;
    }
  }
  fromHsl({
    h,
    s,
    l,
    a
  }) {
    this._h = h % 360;
    this._s = s;
    this._l = l;
    this.a = typeof a === 'number' ? a : 1;
    if (s <= 0) {
      const rgb = Math.round(l * 255);
      this.r = rgb;
      this.g = rgb;
      this.b = rgb;
    }
    const huePrime = h / 60;
    const chroma = (1 - Math.abs(2 * l - 1)) * s;
    const secondComponent = chroma * (1 - Math.abs(huePrime % 2 - 1));
    this.r = 0;
    this.g = 0;
    this.b = 0;
    if (huePrime >= 0 && huePrime < 1) {
      this.r = chroma;
      this.g = secondComponent;
    } else if (huePrime >= 1 && huePrime < 2) {
      this.r = secondComponent;
      this.g = chroma;
    } else if (huePrime >= 2 && huePrime < 3) {
      this.g = chroma;
      this.b = secondComponent;
    } else if (huePrime >= 3 && huePrime < 4) {
      this.g = secondComponent;
      this.b = chroma;
    } else if (huePrime >= 4 && huePrime < 5) {
      this.r = secondComponent;
      this.b = chroma;
    } else if (huePrime >= 5 && huePrime < 6) {
      this.r = chroma;
      this.b = secondComponent;
    }
    const lightnessModification = l - chroma / 2;
    this.r = Math.round((this.r + lightnessModification) * 255);
    this.g = Math.round((this.g + lightnessModification) * 255);
    this.b = Math.round((this.b + lightnessModification) * 255);
  }
  fromHsv({
    h,
    s,
    v,
    a
  }) {
    this._h = h % 360;
    this._s = s;
    this._v = v;
    this.a = typeof a === 'number' ? a : 1;
    const vv = Math.round(v * 255);
    this.r = vv;
    this.g = vv;
    this.b = vv;
    if (s <= 0) {
      return;
    }
    const hh = h / 60;
    const i = Math.floor(hh);
    const ff = hh - i;
    const p = Math.round(v * (1.0 - s) * 255);
    const q = Math.round(v * (1.0 - s * ff) * 255);
    const t = Math.round(v * (1.0 - s * (1.0 - ff)) * 255);
    switch (i) {
      case 0:
        this.g = t;
        this.b = p;
        break;
      case 1:
        this.r = q;
        this.b = p;
        break;
      case 2:
        this.r = p;
        this.b = t;
        break;
      case 3:
        this.r = p;
        this.g = q;
        break;
      case 4:
        this.r = t;
        this.g = p;
        break;
      case 5:
      default:
        this.g = p;
        this.b = q;
        break;
    }
  }
  fromHslString(trimed) {
    const str = trimed.substring(trimed.indexOf('(') + 1, trimed.indexOf(')'));
    const arr = str.includes(',') ? str.split(',') : str.replace('/', ' ').split(' ').filter(item => item.length > 0);
    const h = parseInt(arr[0]);
    const s = parseFloat(arr[1]) / 100;
    const l = parseFloat(arr[2]) / 100;
    const a = arr[3] ? arr[3].includes('%') ? parseFloat(arr[3]) / 100 : parseFloat(arr[3]) : 1;
    this.fromHsl({
      h,
      s,
      l,
      a
    });
  }
  fromRgbString(trimed) {
    const str = trimed.substring(trimed.indexOf('(') + 1, trimed.indexOf(')'));
    const arr = str.includes(',') ? str.split(',') : str.replace('/', ' ').split(' ').filter(item => item.length > 0);
    this.r = arr[0].includes('%') ? Math.round(parseFloat(arr[0]) / 100 * 255) : parseInt(arr[0]);
    this.g = arr[1].includes('%') ? Math.round(parseFloat(arr[1]) / 100 * 255) : parseInt(arr[1]);
    this.b = arr[2].includes('%') ? Math.round(parseFloat(arr[2]) / 100 * 255) : parseInt(arr[2]);
    this.a = arr[3] ? arr[3].includes('%') ? parseFloat(arr[3]) / 100 : parseFloat(arr[3]) : 1;
  }
}
exports.FastColor = FastColor;